import { IEncodeFeature } from '@antv/l7-core';
/**
 * 计算2D 填充点图顶点
 * @param feature 映射feature
 */
export declare function PointFillTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 * 计算2D 填充点图顶点 (地球模式)
 * @param feature 映射feature
 */
export declare function GlobelPointFillTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 * 计算3D 拉伸点图
 * @param feature 映射feature
 */
export declare function PointExtrudeTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    normals: number[] | undefined;
    size: number;
};
/**
 * 计算图片标注
 * @param feature 映射feature
 */
export declare function PointImageTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 * 线三角化
 * @param feature 映射feature
 */
export declare function LineTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    normals: number[];
    indexes: number[];
    size: number;
};
export declare function SimpleLineTriangulation(feature: IEncodeFeature): {
    vertices: any[];
    indices: never[];
    normals: never[];
    size: number;
    count: number;
};
export declare function polygonTriangulation(feature: IEncodeFeature): {
    indices: number[];
    vertices: number[];
    size: number;
};
export declare function polygonTriangulationWithCenter(feature: IEncodeFeature): {
    indices: number[];
    vertices: number[];
    size: number;
};
export declare function PolygonExtrudeTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    normals: number[] | undefined;
    size: number;
};
export declare function HeatmapGridTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 * 图片图层顶点构造
 * @param feature 数据
 */
export declare function RasterImageTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 *  计算3D弧线顶点
 * @param feature 映射数据
 * @param segNum 弧线线段数
 */
export declare function LineArcTriangulation(feature: IEncodeFeature, segmentNumber?: number): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 * 构建热力图密度图的顶点
 * @param feature
 * @returns
 */
export declare function HeatmapTriangulation(feature: IEncodeFeature): {
    vertices: number[];
    indices: number[];
    size: number;
};
/**
 * 构建地球三角网格
 * @returns
 */
export declare function earthTriangulation(): {
    vertices: number[];
    indices: number[];
    size: number;
    normals: number[];
};
export declare function earthOuterTriangulation(): {
    vertices: number[];
    indices: number[];
    size: number;
    normals: number[];
};
