"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _l7Utils = require("@antv/l7-utils");

var _tileLayerManager = require("../manager/tileLayerManager");

var BaseTileLayer = /*#__PURE__*/function () {
  function BaseTileLayer(_ref) {
    var _parentSource$data;

    var parent = _ref.parent,
        rendererService = _ref.rendererService,
        mapService = _ref.mapService,
        layerService = _ref.layerService,
        pickingService = _ref.pickingService,
        transforms = _ref.transforms;
    (0, _classCallCheck2.default)(this, BaseTileLayer);
    (0, _defineProperty2.default)(this, "type", 'baseTile');
    (0, _defineProperty2.default)(this, "initedTileset", false);
    (0, _defineProperty2.default)(this, "pickColors", {
      select: null,
      active: null
    });
    var parentSource = parent.getSource();

    var _ref2 = (parentSource === null || parentSource === void 0 ? void 0 : (_parentSource$data = parentSource.data) === null || _parentSource$data === void 0 ? void 0 : _parentSource$data.tilesetOptions) || {},
        sourceLayer = _ref2.sourceLayer,
        coords = _ref2.coords,
        featureId = _ref2.featureId;

    this.sourceLayer = sourceLayer;
    this.parent = parent;
    this.mapService = mapService;
    this.layerService = layerService;
    this.tileLayerManager = new _tileLayerManager.TileLayerManager(parent, mapService, rendererService, pickingService, layerService, transforms);
    this.initTileSetManager();
    this.bindSubLayerEvent();
    this.bindSubLayerPick();
    this.scaleField = this.parent.getScaleOptions();
  }
  /**
   * 渲染瓦片的图层
   */


  (0, _createClass2.default)(BaseTileLayer, [{
    key: "children",
    get: function get() {
      return this.tileLayerManager.children;
    }
  }, {
    key: "render",
    value: function render() {
      if (this.tileLayerManager) {
        this.tileLayerManager.render();
      }
    }
  }, {
    key: "clearPick",
    value: function clearPick(type) {
      if (type === 'mousemove') {
        this.tileLayerManager.tilePickManager.clearPick();
      }
    }
    /**
     * 清除 select 的选中状态
     */

  }, {
    key: "clearPickState",
    value: function clearPickState() {
      var _this = this;

      this.children.filter(function (child) {
        return child.inited && child.isVisible();
      }).filter(function (child) {
        return child.getCurrentSelectedId() !== null;
      }).map(function (child) {
        _this.selectFeature(child, new Uint8Array([0, 0, 0, 0]));

        child.setCurrentSelectedId(null);
      });
    }
    /**
     * 瓦片图层独立的拾取逻辑
     * @param target
     * @returns
     */

  }, {
    key: "pickLayers",
    value: function pickLayers(target) {
      return this.tileLayerManager.pickLayers(target);
    }
  }, {
    key: "tileLoaded",
    value: function tileLoaded(tile) {//
    }
  }, {
    key: "tileError",
    value: function tileError(error) {
      console.warn('error:', error);
    }
  }, {
    key: "tileUnLoad",
    value: function tileUnLoad(tile) {
      this.tileLayerManager.removeChilds(tile.layerIDList, false);
    }
  }, {
    key: "tileUpdate",
    value: function tileUpdate() {
      var _this2 = this;

      // Base Function
      if (!this.tilesetManager) {
        return;
      }

      this.tilesetManager.tiles.filter(function (tile) {
        return tile.isLoaded;
      }).map(function (tile) {
        if (tile.layerIDList.length === 0) {
          var _this2$tileLayerManag = _this2.tileLayerManager.createTile(tile),
              layers = _this2$tileLayerManag.layers,
              layerIDList = _this2$tileLayerManag.layerIDList;

          tile.layerIDList = layerIDList;

          _this2.tileLayerManager.addChilds(layers);
        } else {
          if (!tile.isVisibleChange) {
            return;
          }

          var _layers = _this2.tileLayerManager.getChilds(tile.layerIDList);

          _this2.tileLayerManager.updateLayersConfig(_layers, 'visible', tile.isVisible);
        }
      });

      if (this.tilesetManager.isLoaded) {
        // 将事件抛出，图层上可以使用瓦片
        this.parent.emit('tiles-loaded', this.tilesetManager.currentTiles);
      }
    }
  }, {
    key: "setPickState",
    value: function setPickState(layers) {
      var _this3 = this;

      if (this.pickColors.select) {
        var selectedId = (0, _l7Utils.decodePickingColor)(this.pickColors.select);
        layers.map(function (layer) {
          _this3.selectFeature(layer, _this3.pickColors.select);

          layer.setCurrentSelectedId(selectedId);
        });
      }

      if (this.pickColors.active) {
        var _selectedId = (0, _l7Utils.decodePickingColor)(this.pickColors.active);

        layers.filter(function (layer) {
          return layer.inited && layer.isVisible();
        }).map(function (layer) {
          layer.hooks.beforeHighlight.call(_this3.pickColors.active);
          layer.setCurrentPickId(_selectedId);
        });
      }
    }
  }, {
    key: "bindSubLayerPick",
    value: function bindSubLayerPick() {
      var _this4 = this;

      this.tileLayerManager.tilePickManager.on('pick', function (e) {
        // @ts-ignore
        var _e$pickedColors = (0, _slicedToArray2.default)(e.pickedColors, 3),
            r = _e$pickedColors[0],
            g = _e$pickedColors[1],
            b = _e$pickedColors[2];

        if (e.type === 'click') {
          var restLayers = _this4.children.filter(function (child) {
            return child.inited && child.isVisible() && child.isVector;
          }).filter(function (child) {
            return child !== e.layer;
          });

          _this4.setSelect(restLayers, [r, g, b]);
        } else {
          _this4.setHighlight([r, g, b]);
        }
      });
      this.tileLayerManager.tilePickManager.on('unpick', function () {
        _this4.pickColors.active = null;
      });
    }
  }, {
    key: "setHighlight",
    value: function setHighlight(pickedColors) {
      var pickId = (0, _l7Utils.decodePickingColor)(pickedColors);
      this.pickColors.active = pickedColors;
      this.children.filter(function (child) {
        return child.inited && child.isVisible() && child.isVector;
      }) // Tip: 使用 vectorLayer 上的 pickID 优化高亮操作（过滤重复操作）
      // @ts-ignore
      .filter(function (child) {
        return child.getPickID() !== pickId;
      }).map(function (child) {
        // @ts-ignore
        child.setPickID(pickId);
        child.hooks.beforeHighlight.call(pickedColors);
      });
    }
  }, {
    key: "setSelect",
    value: function setSelect(layers, pickedColors) {
      var _this5 = this;

      var selectedId = (0, _l7Utils.decodePickingColor)(pickedColors);
      layers.map(function (layer) {
        if (layer.getCurrentSelectedId() === null || selectedId !== layer.getCurrentSelectedId()) {
          _this5.selectFeature(layer, pickedColors);

          layer.setCurrentSelectedId(selectedId);
          _this5.pickColors.select = pickedColors;
        } else {
          _this5.selectFeature(layer, new Uint8Array([0, 0, 0, 0])); // toggle select


          layer.setCurrentSelectedId(null);
          _this5.pickColors.select = null;
        }
      }); // unselect normal layer

      var renderList = this.layerService.getRenderList();
      renderList.filter(function (layer) {
        return layer.inited && !layer.isVector && layer.isVisible() && layer.needPick('click');
      }).filter(function (layer) {
        return layer.getCurrentSelectedId() !== null;
      }).map(function (layer) {
        _this5.selectFeature(layer, new Uint8Array([0, 0, 0, 0]));

        layer.setCurrentSelectedId(null);
      });
    }
  }, {
    key: "selectFeature",
    value: function selectFeature(layer, pickedColors) {
      // @ts-ignore
      var _pickedColors = (0, _slicedToArray2.default)(pickedColors, 3),
          r = _pickedColors[0],
          g = _pickedColors[1],
          b = _pickedColors[2];

      layer.hooks.beforeSelect.call([r, g, b]);
    }
  }, {
    key: "bindSubLayerEvent",
    value: function bindSubLayerEvent() {
      var _this6 = this;

      /**
       * layer.on('click', (ev) => {}); // 鼠标左键点击图层事件
       * layer.on('mouseenter', (ev) => {}); // 鼠标进入图层要素
       * layer.on('mousemove', (ev) => {}); // 鼠标在图层上移动时触发
       * layer.on('mouseout', (ev) => {}); // 鼠标移出图层要素时触发
       * layer.on('mouseup', (ev) => {}); // 鼠标在图层上单击抬起时触发
       * layer.on('mousedown', (ev) => {}); // 鼠标在图层上单击按下时触发
       * layer.on('contextmenu', (ev) => {}); // 图层要素点击右键菜单
       *
       *  鼠标在图层外的事件
       * layer.on('unclick', (ev) => {}); // 图层外点击
       * layer.on('unmousemove', (ev) => {}); // 图层外移动
       * layer.on('unmouseup', (ev) => {}); // 图层外鼠标抬起
       * layer.on('unmousedown', (ev) => {}); // 图层外单击按下时触发
       * layer.on('uncontextmenu', (ev) => {}); // 图层外点击右键
       * layer.on('unpick', (ev) => {}); // 图层外的操作的所有事件
       */
      this.parent.on('subLayerClick', function (e) {
        _this6.parent.emit('click', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerMouseMove', function (e) {
        return _this6.parent.emit('mousemove', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerMouseUp', function (e) {
        return _this6.parent.emit('mouseup', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerMouseEnter', function (e) {
        return _this6.parent.emit('mouseenter', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerMouseOut', function (e) {
        return _this6.parent.emit('mouseout', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerMouseDown', function (e) {
        return _this6.parent.emit('mousedown', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerContextmenu', function (e) {
        return _this6.parent.emit('contextmenu', (0, _objectSpread2.default)({}, e));
      }); // vector layer 图层外事件

      this.parent.on('subLayerUnClick', function (e) {
        return _this6.parent.emit('unclick', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerUnMouseMove', function (e) {
        return _this6.parent.emit('unmousemove', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerUnMouseUp', function (e) {
        return _this6.parent.emit('unmouseup', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerUnMouseDown', function (e) {
        return _this6.parent.emit('unmousedown', (0, _objectSpread2.default)({}, e));
      });
      this.parent.on('subLayerUnContextmenu', function (e) {
        return _this6.parent.emit('uncontextmenu', (0, _objectSpread2.default)({}, e));
      });
    }
  }, {
    key: "initTileSetManager",
    value: function initTileSetManager() {
      var _this$tilesetManager;

      var source = this.parent.getSource();
      this.tilesetManager = source.tileset;

      if (!this.initedTileset) {
        this.bindTilesetEvent();
        this.initedTileset = true;
      }

      var _this$getCurrentView = this.getCurrentView(),
          latLonBounds = _this$getCurrentView.latLonBounds,
          zoom = _this$getCurrentView.zoom;

      (_this$tilesetManager = this.tilesetManager) === null || _this$tilesetManager === void 0 ? void 0 : _this$tilesetManager.update(zoom, latLonBounds);
    }
  }, {
    key: "mapchange",
    value: function mapchange() {
      var _this$tilesetManager2;

      var _this$getCurrentView2 = this.getCurrentView(),
          latLonBounds = _this$getCurrentView2.latLonBounds,
          zoom = _this$getCurrentView2.zoom;

      if (this.mapService.version === 'GAODE1.x') {
        var _this$parent$getLayer = this.parent.getLayerConfig(),
            visible = _this$parent$getLayer.visible;

        if (zoom < 3 && visible) {
          this.parent.updateLayerConfig({
            visible: false
          });
          this.layerService.updateLayerRenderList();
        } else if (zoom >= 3 && !visible) {
          this.parent.updateLayerConfig({
            visible: true
          });
          this.layerService.updateLayerRenderList();
        }
      }

      if (this.lastViewStates && this.lastViewStates.zoom === zoom && this.lastViewStates.latLonBounds.toString() === latLonBounds.toString()) {
        return;
      }

      this.lastViewStates = {
        zoom: zoom,
        latLonBounds: latLonBounds
      };

      if (this.timer) {
        clearTimeout(this.timer);
        this.timer = null;
      } // this.timer = setTimeout(() => {


      (_this$tilesetManager2 = this.tilesetManager) === null || _this$tilesetManager2 === void 0 ? void 0 : _this$tilesetManager2.update(zoom, latLonBounds); // }, 250);
    }
  }, {
    key: "bindTilesetEvent",
    value: function bindTilesetEvent() {
      var _this7 = this;

      if (!this.tilesetManager) {
        return;
      } // 瓦片数据加载成功


      this.tilesetManager.on('tile-loaded', function (tile) {// todo: 将事件抛出，图层上可以监听使用
      }); // 瓦片数据从缓存删除或被执行重新加载

      this.tilesetManager.on('tile-unload', function (tile) {
        // todo: 将事件抛出，图层上可以监听使用
        _this7.tileUnLoad(tile);
      }); // 瓦片数据加载失败

      this.tilesetManager.on('tile-error', function (error, tile) {
        // todo: 将事件抛出，图层上可以监听使用
        _this7.tileError(error);
      }); // 瓦片显隐状态更新

      this.tilesetManager.on('tile-update', function () {
        _this7.tileUpdate();
      }); // 地图视野发生改变

      this.mapService.on('zoomend', function () {
        return _this7.mapchange();
      });
      this.mapService.on('moveend', function () {
        return _this7.mapchange();
      });
    }
  }, {
    key: "getCurrentView",
    value: function getCurrentView() {
      var bounds = this.mapService.getBounds();
      var latLonBounds = [bounds[0][0], bounds[0][1], bounds[1][0], bounds[1][1]];
      var zoom = this.mapService.getZoom();
      return {
        latLonBounds: latLonBounds,
        zoom: zoom
      };
    }
  }]);
  return BaseTileLayer;
}();

exports.default = BaseTileLayer;